@@ -27,7 +27,9 @@ enum {
   MOUSEPAD_RESPONSE_CLEAR,
   MOUSEPAD_RESPONSE_CLOSE,
   MOUSEPAD_RESPONSE_DONT_SAVE,
+  MOUSEPAD_RESPONSE_ENTRY_CHANGED,
   MOUSEPAD_RESPONSE_FIND,
+  MOUSEPAD_RESPONSE_REVERSE_FIND,
   MOUSEPAD_RESPONSE_JUMP_TO,
   MOUSEPAD_RESPONSE_OK,
   MOUSEPAD_RESPONSE_OVERWRITE,
@@ -35,11 +37,8 @@ enum {
   MOUSEPAD_RESPONSE_REVERT,
   MOUSEPAD_RESPONSE_SAVE,
   MOUSEPAD_RESPONSE_SAVE_AS,
-  MOUSEPAD_RESPONSE_CHECK_ENTRY
 };
 
-GtkWidget *mousepad_dialogs_image_button        (const gchar   *stock_id,
-                                                 const gchar   *label);
 
 void       mousepad_dialogs_show_about          (GtkWindow     *parent);
 
@@ -66,6 +65,15 @@ gint       mousepad_dialogs_externally_modified (GtkWindow     *parent);
 
 gint       mousepad_dialogs_revert              (GtkWindow     *parent);
 
+gint       mousepad_dialogs_save_as             (GtkWindow     *parent,
+                                                 const gchar   *current_filename,
+                                                 const gchar   *last_save_location,
+                                                 gchar        **filename);
+
+gint       mousepad_dialogs_open                (GtkWindow     *parent,
+                                                 const gchar   *filename,
+                                                 GSList       **filenames);
+
 G_END_DECLS
 
 #endif /* !__MOUSEPAD_DIALOGS_H__ */
