@@ -15,7 +15,6 @@
  */
 
 #include <mousepad/mousepad-private.h>
-#include <mousepad/mousepad-gtkcompat.h>
 #include <mousepad/mousepad-close-button.h>
 #include <mousepad/mousepad-settings.h>
 #include <mousepad/mousepad-util.h>
@@ -24,11 +23,7 @@
 #include <mousepad/mousepad-view.h>
 #include <mousepad/mousepad-window.h>
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 #include <gtksourceview/gtksource.h>
-#else
-#include <gtksourceview/gtksourcebuffer.h>
-#endif
 
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
@@ -85,9 +80,10 @@ struct _MousepadDocumentPrivate
 {
   GtkScrolledWindow __parent__;
 
-  /* the tab label and ebox */
+  /* the tab label, ebox and CSS provider */
   GtkWidget           *ebox;
   GtkWidget           *label;
+  GtkCssProvider      *css_provider;
 
   /* utf-8 valid document names */
   gchar               *utf8_filename;
@@ -175,24 +171,26 @@ mousepad_document_init (MousepadDocument *document)
   document->priv->utf8_filename = NULL;
   document->priv->utf8_basename = NULL;
   document->priv->label = NULL;
+  document->priv->css_provider = gtk_css_provider_new ();
 
   /* setup the scolled window */
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (document), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (document),
+                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (document), GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_hadjustment (GTK_SCROLLED_WINDOW (document), NULL);
   gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW (document), NULL);
 
-  /* create a textbuffer */
+  /* create a textbuffer and associated search context */
   document->buffer = GTK_TEXT_BUFFER (gtk_source_buffer_new (NULL));
+  document->search_context = gtk_source_search_context_new (GTK_SOURCE_BUFFER (document->buffer), NULL);
+  gtk_source_search_context_set_highlight (document->search_context, FALSE);
 
   /* initialize the file */
   document->file = mousepad_file_new (document->buffer);
 
   /* connect signals to the file */
-  g_signal_connect_swapped (G_OBJECT (document->file), "filename-changed", G_CALLBACK (mousepad_document_filename_changed), document);
-
-  /* create the highlight tag */
-  document->tag = gtk_text_buffer_create_tag (document->buffer, NULL, "background", "#ffff78", NULL);
+  g_signal_connect_swapped (G_OBJECT (document->file), "filename-changed",
+                            G_CALLBACK (mousepad_document_filename_changed), document);
 
   /* setup the textview */
   document->textview = g_object_new (MOUSEPAD_TYPE_VIEW, "buffer", document->buffer, NULL);
@@ -204,13 +202,20 @@ mousepad_document_init (MousepadDocument *document)
   gtk_target_list_add_table (target_list, drop_targets, G_N_ELEMENTS (drop_targets));
 
   /* attach signals to the text view and buffer */
-  g_signal_connect (G_OBJECT (document->buffer), "notify::cursor-position", G_CALLBACK (mousepad_document_notify_cursor_position), document);
-  g_signal_connect (G_OBJECT (document->buffer), "notify::has-selection", G_CALLBACK (mousepad_document_notify_has_selection), document);
-  g_signal_connect_swapped (G_OBJECT (document->buffer), "modified-changed", G_CALLBACK (mousepad_document_label_color), document);
-  g_signal_connect_swapped (G_OBJECT (document->file), "readonly-changed", G_CALLBACK (mousepad_document_label_color), document);
-  g_signal_connect (G_OBJECT (document->textview), "notify::overwrite", G_CALLBACK (mousepad_document_notify_overwrite), document);
-  g_signal_connect (G_OBJECT (document->textview), "drag-data-received", G_CALLBACK (mousepad_document_drag_data_received), document);
-  g_signal_connect (G_OBJECT (document->buffer), "notify::language", G_CALLBACK (mousepad_document_notify_language), document);
+  g_signal_connect (G_OBJECT (document->buffer), "notify::cursor-position",
+                    G_CALLBACK (mousepad_document_notify_cursor_position), document);
+  g_signal_connect (G_OBJECT (document->buffer), "notify::has-selection",
+                    G_CALLBACK (mousepad_document_notify_has_selection), document);
+  g_signal_connect_swapped (G_OBJECT (document->buffer), "modified-changed",
+                            G_CALLBACK (mousepad_document_label_color), document);
+  g_signal_connect_swapped (G_OBJECT (document->file), "readonly-changed",
+                            G_CALLBACK (mousepad_document_label_color), document);
+  g_signal_connect (G_OBJECT (document->textview), "notify::overwrite",
+                    G_CALLBACK (mousepad_document_notify_overwrite), document);
+  g_signal_connect (G_OBJECT (document->textview), "drag-data-received",
+                    G_CALLBACK (mousepad_document_drag_data_received), document);
+  g_signal_connect (G_OBJECT (document->buffer), "notify::language",
+                    G_CALLBACK (mousepad_document_notify_language), document);
 }
 
 
@@ -223,12 +228,14 @@ mousepad_document_finalize (GObject *object)
   /* cleanup */
   g_free (document->priv->utf8_filename);
   g_free (document->priv->utf8_basename);
+  g_object_unref (document->priv->css_provider);
 
   /* release the file */
   g_object_unref (G_OBJECT (document->file));
 
-  /* release the buffer reference */
+  /* release the buffer reference and the search context reference */
   g_object_unref (G_OBJECT (document->buffer));
+  g_object_unref (G_OBJECT (document->search_context));
 
   (*G_OBJECT_CLASS (mousepad_document_parent_class)->finalize) (object);
 }
@@ -348,7 +355,8 @@ mousepad_document_drag_data_received (GtkWidget        *widget,
 
   /* emit the drag-data-received signal from the document when a tab or uri has been dropped */
   if (info == TARGET_TEXT_URI_LIST || info == TARGET_GTK_NOTEBOOK_TAB)
-    g_signal_emit_by_name (G_OBJECT (document), "drag-data-received", context, x, y, selection_data, info, drag_time);
+    g_signal_emit_by_name (G_OBJECT (document), "drag-data-received", context,
+                           x, y, selection_data, info, drag_time);
 }
 
 
@@ -357,7 +365,9 @@ static void
 mousepad_document_filename_changed (MousepadDocument *document,
                                     const gchar      *filename)
 {
-  gchar *utf8_filename, *utf8_basename;
+  gchar       *utf8_filename, *utf8_short_filename, *utf8_basename;
+  const gchar *home;
+  size_t       home_len;
 
   g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
   g_return_if_fail (filename != NULL);
@@ -367,6 +377,15 @@ mousepad_document_filename_changed (MousepadDocument *document,
 
   if (G_LIKELY (utf8_filename))
     {
+      /* create a shorter display filename: replace $HOME with a tilde if user is not root */
+      if (geteuid () && (home = g_get_home_dir ()) && (home_len = strlen (home))
+          && g_str_has_prefix (utf8_filename, home))
+        {
+          utf8_short_filename = g_strconcat ("~", utf8_filename + home_len, NULL);
+          g_free (utf8_filename);
+          utf8_filename = utf8_short_filename;
+        }
+
       /* create the display name */
       utf8_basename = g_filename_display_basename (utf8_filename);
 
@@ -398,9 +417,7 @@ mousepad_document_filename_changed (MousepadDocument *document,
 static void
 mousepad_document_label_color (MousepadDocument *document)
 {
-  GdkColor  green = {0, 0x0000, 0x9999, 0x0000};
-  GdkColor  red   = {0, 0xffff, 0x0000, 0x0000};
-  GdkColor *color;
+  GtkStyleContext *context;
 
   g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
   g_return_if_fail (GTK_IS_TEXT_BUFFER (document->buffer));
@@ -408,17 +425,24 @@ mousepad_document_label_color (MousepadDocument *document)
 
   if (document->priv->label)
     {
-      /* label color */
-      if (gtk_text_buffer_get_modified (document->buffer))
-        color = &red;
-      else if (mousepad_file_get_read_only (document->file))
-        color = &green;
-      else
-        color = NULL;
+      context = gtk_widget_get_style_context (document->priv->label);
 
-      /* update colors */
-      gtk_widget_modify_fg (document->priv->label, GTK_STATE_NORMAL, color);
-      gtk_widget_modify_fg (document->priv->label, GTK_STATE_ACTIVE, color);
+      /* grey out the label text */
+      if (mousepad_file_get_read_only (document->file))
+        gtk_style_context_add_class (context, GTK_STYLE_CLASS_DIM_LABEL);
+      else
+        gtk_style_context_remove_class (context, GTK_STYLE_CLASS_DIM_LABEL);
+
+      /* change the label text color */
+      if (gtk_text_buffer_get_modified (document->buffer))
+        {
+          gtk_css_provider_load_from_data (document->priv->css_provider,
+                                           "label { color: red; }", -1, NULL);
+          gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (document->priv->css_provider),
+                                          GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+        }
+      else
+        gtk_style_context_remove_provider (context, GTK_STYLE_PROVIDER (document->priv->css_provider));
     }
 }
 
@@ -469,21 +493,22 @@ mousepad_document_send_signals (MousepadDocument *document)
 GtkWidget *
 mousepad_document_get_tab_label (MousepadDocument *document)
 {
-  GtkWidget  *hbox, *align;
-  GtkWidget  *button;
+  GtkWidget  *hbox, *button;
 
   /* create the box */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_show (hbox);
 
   /* the ebox */
-  document->priv->ebox = g_object_new (GTK_TYPE_EVENT_BOX, "border-width", 2, "visible-window", FALSE, NULL);
+  document->priv->ebox = g_object_new (GTK_TYPE_EVENT_BOX, "border-width", 2,
+                                       "visible-window", FALSE, NULL);
   gtk_box_pack_start (GTK_BOX (hbox), document->priv->ebox, TRUE, TRUE, 0);
   gtk_widget_set_tooltip_text (document->priv->ebox, document->priv->utf8_filename);
   gtk_widget_show (document->priv->ebox);
 
   /* create the label */
   document->priv->label = gtk_label_new (mousepad_document_get_basename (document));
+  gtk_label_set_ellipsize (GTK_LABEL (document->priv->label), PANGO_ELLIPSIZE_MIDDLE);
   gtk_container_add (GTK_CONTAINER (document->priv->ebox), document->priv->label);
   gtk_widget_show (document->priv->label);
 
@@ -491,16 +516,14 @@ mousepad_document_get_tab_label (MousepadDocument *document)
   mousepad_document_label_color (document);
 
   /* create the button */
-  align = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
   button = mousepad_close_button_new ();
-  gtk_container_add (GTK_CONTAINER (align), button);
   gtk_widget_show (button);
 
   /* pack button, add signal and tooltip */
   gtk_widget_set_tooltip_text (button, _("Close this tab"));
-  gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
-  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (mousepad_document_tab_button_clicked), document);
-  gtk_widget_show (align);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  g_signal_connect (G_OBJECT (button), "clicked",
+                    G_CALLBACK (mousepad_document_tab_button_clicked), document);
 
   return hbox;
 }
