@@ -68,13 +68,17 @@ enum
 #define MOUSEPAD_HAS_FLAG(flags,flag)   (((flags) & (flag)) != 0)
 
 /* for personal testing */
-#define TIMER_START    GTimer *__FUNCTION__timer = g_timer_new();
-#define TIMER_SPLIT    g_print ("%s: %.2f ms\n", G_STRLOC, g_timer_elapsed (__FUNCTION__timer, NULL) * 1000);
-#define TIMER_STOP     TIMER_SPLIT g_timer_destroy (__FUNCTION__timer);
-#define PRINT_LOCATION g_print ("%s\n", G_STRLOC);
+#define TIMER_START     GTimer *__FUNCTION__timer = g_timer_new ();
+#define TIMER_SPLIT     g_printerr ("%s: %.2f ms\n", G_STRLOC, \
+                                    g_timer_elapsed (__FUNCTION__timer, NULL) * 1000);
+#define TIMER_STOP      TIMER_SPLIT g_timer_destroy (__FUNCTION__timer);
+
+#define PRINT_LOCATION  g_printerr ("%s\n", G_STRLOC);
+#define TRACE(fmt, var) g_printerr ("%s:%s: ", G_STRLOC, #var); g_printerr (fmt, var); g_printerr ("\n");
 
 /* optimize the properties */
-#define MOUSEPAD_PARAM_READWRITE (G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB)
+#define MOUSEPAD_PARAM_READWRITE (G_PARAM_READWRITE | G_PARAM_STATIC_NAME \
+                                  | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB)
 
 /* support for canonical strings and quarks */
 #define I_(string)  (g_intern_static_string (string))
@@ -83,8 +87,10 @@ enum
 #define mousepad_object_set_data(object,key,data)              (g_object_set_qdata ((object), \
                                                                 g_quark_from_static_string (key), (data)))
 #define mousepad_object_set_data_full(object,key,data,destroy) (g_object_set_qdata_full ((object), \
-                                                                g_quark_from_static_string (key), (data), (GDestroyNotify) (destroy)))
-#define mousepad_object_get_data(object,key)                   (g_object_get_qdata ((object), g_quark_try_string (key)))
+                                                                g_quark_from_static_string (key), (data), \
+                                                                (GDestroyNotify) (destroy)))
+#define mousepad_object_get_data(object,key)                   (g_object_get_qdata ((object), \
+                                                                g_quark_try_string (key)))
 
 /* avoid trivial g_value_get_*() function calls */
 #ifdef NDEBUG
@@ -112,7 +118,7 @@ enum
 #undef G_LIKELY
 #undef G_UNLIKELY
 
-#if defined(NDEBUG) && defined(__GNUC__) && (__GNUC__ > 2)
+#if defined (NDEBUG) && G_GNUC_CHECK_VERSION (3, 0)
 #define G_LIKELY(expr) (__builtin_expect (!!(expr), TRUE))
 #define G_UNLIKELY(expr) (__builtin_expect (!!(expr), FALSE))
 #else
@@ -142,6 +148,17 @@ mousepad_disconnect_by_func_ (gpointer  instance,
 #define mousepad_disconnect_by_func(instance, callback, data) \
   mousepad_disconnect_by_func_ (instance, G_CALLBACK (callback), data)
 
+/*
+ * To suppress warnings from GCC 8 onwards with -Wextra or -Wcast-function-type enabled
+ * about the function types being incompatible.
+ * See https://developer.gnome.org/glib/stable/glib-The-Main-Event-Loop.html#G-SOURCE-FUNC:CAPS
+ * and https://gitlab.xfce.org/apps/mousepad/-/merge_requests/25#note_15876
+ */
+#if !GLIB_CHECK_VERSION (2, 58, 0)
+# define G_SOURCE_FUNC(f) ((GSourceFunc) (void (*)(void)) (f))
+#endif
+
+
 G_END_DECLS
 
 #endif /* !__MOUSEPAD_PRIVATE_H__ */
