@@ -21,20 +21,23 @@
 #include <mousepad/mousepad-dialogs.h>
 #include <mousepad/mousepad-util.h>
 #include <mousepad/mousepad-marshal.h>
-#include <mousepad/mousepad-gtkcompat.h>
 
 
 
-static void                 mousepad_replace_dialog_unrealize           (GtkWidget             *widget);
-static void                 mousepad_replace_dialog_finalize            (GObject               *object);
-static void                 mousepad_replace_dialog_response            (GtkWidget             *widget,
+static void              mousepad_replace_dialog_unrealize              (GtkWidget             *widget);
+static void              mousepad_replace_dialog_finalize               (GObject               *object);
+static void              mousepad_replace_dialog_response               (GtkWidget             *widget,
                                                                          gint                   response_id);
-static void                 mousepad_replace_dialog_changed             (MousepadReplaceDialog *dialog);
-static void                 mousepad_replace_dialog_settings_changed    (MousepadReplaceDialog *dialog,
+static void              mousepad_replace_dialog_changed                (MousepadReplaceDialog *dialog);
+static void              mousepad_replace_dialog_settings_changed       (MousepadReplaceDialog *dialog,
                                                                          gchar                 *key,
                                                                          GSettings             *settings);
-static void                 mousepad_replace_dialog_history_combo_box   (GtkComboBoxText       *combo_box);
-static void                 mousepad_replace_dialog_history_insert_text (const gchar           *text);
+static void              mousepad_replace_dialog_entry_activate         (GtkWidget             *entry,
+                                                                         MousepadReplaceDialog *dialog);
+static void              mousepad_replace_dialog_entry_reverse_activate (GtkWidget             *entry,
+                                                                         MousepadReplaceDialog *dialog);
+static void              mousepad_replace_dialog_history_combo_box      (GtkComboBoxText       *combo_box);
+static void              mousepad_replace_dialog_history_insert_text    (const gchar           *text);
 
 
 
@@ -48,12 +51,15 @@ struct _MousepadReplaceDialog
   GtkDialog __parent__;
 
   /* dialog widgets */
-  GtkWidget           *search_entry;
-  GtkWidget           *replace_entry;
-  GtkWidget           *find_button;
-  GtkWidget           *replace_button;
-  GtkWidget           *search_location_combo;
-  GtkWidget           *hits_label;
+  GtkWidget *search_entry;
+  GtkWidget *replace_entry;
+  GtkWidget *find_button;
+  GtkWidget *replace_button;
+  GtkWidget *search_location_combo;
+  GtkWidget *hits_label;
+
+  /* dialog buttons sensitivity */
+  gboolean   sensitive;
 };
 
 enum
@@ -66,8 +72,7 @@ enum
 enum
 {
   DIRECTION_UP = 0,
-  DIRECTION_DOWN,
-  DIRECTION_BOTH
+  DIRECTION_DOWN
 };
 
 enum
@@ -89,8 +94,9 @@ G_DEFINE_TYPE (MousepadReplaceDialog, mousepad_replace_dialog, GTK_TYPE_DIALOG)
 static void
 mousepad_replace_dialog_class_init (MousepadReplaceDialogClass *klass)
 {
-  GObjectClass   *gobject_class;
+  GObjectClass   *gobject_class, *entry_class;
   GtkWidgetClass *gtkwidget_class;
+  GtkBindingSet  *binding_set;
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = mousepad_replace_dialog_finalize;
@@ -107,6 +113,23 @@ mousepad_replace_dialog_class_init (MousepadReplaceDialogClass *klass)
                   G_TYPE_INT, 3,
                   MOUSEPAD_TYPE_SEARCH_FLAGS,
                   G_TYPE_STRING, G_TYPE_STRING);
+
+  /* add a reverse-activate signal to GtkEntry */
+  entry_class = g_type_class_ref (GTK_TYPE_ENTRY);
+  if (G_LIKELY (g_signal_lookup ("reverse-activate", GTK_TYPE_ENTRY) == 0))
+    {
+      /* install the signal */
+      g_signal_new ("reverse-activate",
+                   GTK_TYPE_ENTRY,
+                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                   0, NULL, NULL,
+                   g_cclosure_marshal_VOID__VOID,
+                   G_TYPE_NONE, 0);
+      binding_set = gtk_binding_set_by_class (entry_class);
+      gtk_binding_entry_add_signal (binding_set, GDK_KEY_Return, GDK_SHIFT_MASK, "reverse-activate", 0);
+      gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Enter, GDK_SHIFT_MASK, "reverse-activate", 0);
+    }
+  g_type_class_unref (entry_class);
 }
 
 
@@ -119,10 +142,10 @@ mousepad_replace_dialog_bind_setting (MousepadReplaceDialog *dialog,
 {
   mousepad_setting_bind (path, object, property, G_SETTINGS_BIND_DEFAULT);
 
-  mousepad_setting_connect (path,
-                            G_CALLBACK (mousepad_replace_dialog_settings_changed),
-                            dialog,
-                            G_CONNECT_SWAPPED);
+  mousepad_setting_connect_object (path,
+                                   G_CALLBACK (mousepad_replace_dialog_settings_changed),
+                                   dialog,
+                                   G_CONNECT_SWAPPED);
 }
 
 
@@ -130,29 +153,41 @@ mousepad_replace_dialog_bind_setting (MousepadReplaceDialog *dialog,
 static void
 mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
 {
-  GtkWidget    *vbox, *hbox, *combo, *label, *check;
+  GtkWidget    *button;
+  GtkWidget    *area, *vbox, *hbox;
+  GtkWidget    *combo;
+  GtkWidget    *label;
+  GtkWidget    *check;
   GtkSizeGroup *size_group;
 
   /* set dialog properties */
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Replace"));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Find and Replace"));
   gtk_window_set_default_size (GTK_WINDOW (dialog), 400, -1);
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (mousepad_replace_dialog_response), NULL);
+  g_signal_connect (G_OBJECT (dialog), "response",
+                    G_CALLBACK (mousepad_replace_dialog_response), NULL);
 
   /* dialog buttons */
-  dialog->find_button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_FIND, MOUSEPAD_RESPONSE_FIND);
-  dialog->replace_button = mousepad_util_image_button (GTK_STOCK_FIND_AND_REPLACE, _("_Replace"));
-  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), dialog->replace_button, MOUSEPAD_RESPONSE_REPLACE);
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, MOUSEPAD_RESPONSE_CLOSE);
+  dialog->find_button = mousepad_util_image_button ("edit-find", _("_Find"));
+  gtk_widget_set_can_default (dialog->find_button, TRUE);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+                                dialog->find_button, MOUSEPAD_RESPONSE_FIND);
+  dialog->replace_button = mousepad_util_image_button ("edit-find-replace", _("_Replace"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+                                dialog->replace_button, MOUSEPAD_RESPONSE_REPLACE);
+  button = mousepad_util_image_button ("window-close", _("_Close"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_FIND);
 
   /* create main vertical box */
-  vbox = g_object_new (GTK_TYPE_VBOX, "border-width", 6, "spacing", 4, NULL);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dialog))), vbox, TRUE, TRUE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 4);
+  area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_pack_start (GTK_BOX (area), vbox, TRUE, TRUE, 6);
   gtk_widget_show (vbox);
 
   /* horizontal box for search string */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+  gtk_widget_set_margin_start (GTK_WIDGET (hbox), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (hbox), 6);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
   gtk_widget_show (hbox);
 
@@ -162,28 +197,37 @@ mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
   label = gtk_label_new_with_mnemonic (_("_Search for:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_size_group_add_widget (size_group, label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
   gtk_widget_show (label);
 
   combo = gtk_combo_box_text_new_with_entry ();
   mousepad_replace_dialog_history_combo_box (GTK_COMBO_BOX_TEXT (combo));
   gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label), combo);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
   gtk_widget_show (combo);
 
   /* store as an entry widget */
   dialog->search_entry = gtk_bin_get_child (GTK_BIN (combo));
-  g_signal_connect_swapped (G_OBJECT (dialog->search_entry), "changed", G_CALLBACK (mousepad_replace_dialog_changed), dialog);
+  g_signal_connect_swapped (G_OBJECT (dialog->search_entry), "changed",
+                            G_CALLBACK (mousepad_replace_dialog_changed), dialog);
+  g_signal_connect (G_OBJECT (dialog->search_entry), "activate",
+                    G_CALLBACK (mousepad_replace_dialog_entry_activate), dialog);
+  g_signal_connect (G_OBJECT (dialog->search_entry), "reverse-activate",
+                    G_CALLBACK (mousepad_replace_dialog_entry_reverse_activate), dialog);
 
   /* horizontal box for replace string */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+  gtk_widget_set_margin_start (GTK_WIDGET (hbox), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (hbox), 6);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
   gtk_widget_show (hbox);
 
   label = gtk_label_new_with_mnemonic (_("Replace _with:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_size_group_add_widget (size_group, label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
   gtk_widget_show (label);
 
   combo = gtk_combo_box_text_new_with_entry ();
@@ -196,45 +240,74 @@ mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
 
   /* search direction */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+  gtk_widget_set_margin_start (GTK_WIDGET (hbox), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (hbox), 6);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label), combo);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
   gtk_widget_show (hbox);
 
   label = gtk_label_new_with_mnemonic (_("Search _direction:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_size_group_add_widget (size_group, label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
   gtk_widget_show (label);
 
   combo = gtk_combo_box_text_new ();
   gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label), combo);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Up"));
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Down"));
-  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Both"));
   gtk_widget_show (combo);
 
-  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_DIRECTION, combo, "active");
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_DIRECTION,
+                                        combo, "active");
 
   /* release size group */
   g_object_unref (G_OBJECT (size_group));
 
+  /* wrap around */
+  check = gtk_check_button_new_with_mnemonic (_("_Wrap around"));
+  gtk_box_pack_start (GTK_BOX (hbox), check, FALSE, FALSE, 0);
+  gtk_widget_show (check);
+
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_WRAP_AROUND,
+                                        check, "active");
+
   /* case sensitive */
-  check = gtk_check_button_new_with_mnemonic (_("Case sensi_tive"));
+  check = gtk_check_button_new_with_mnemonic (_("Match _case"));
+  gtk_widget_set_margin_start (GTK_WIDGET (check), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (check), 6);
   gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, FALSE, 0);
   gtk_widget_show (check);
 
-  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_MATCH_CASE, check, "active");
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_MATCH_CASE,
+                                        check, "active");
 
   /* match whole word */
   check = gtk_check_button_new_with_mnemonic (_("_Match whole word"));
+  gtk_widget_set_margin_start (GTK_WIDGET (check), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (check), 6);
   gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, FALSE, 0);
   gtk_widget_show (check);
 
-  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_MATCH_WHOLE_WORD, check, "active");
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_MATCH_WHOLE_WORD,
+                                        check, "active");
+
+  /* enable regex search */
+  check = gtk_check_button_new_with_mnemonic (_("Regular e_xpression"));
+  gtk_widget_set_margin_start (GTK_WIDGET (check), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (check), 6);
+  gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, FALSE, 0);
+  gtk_widget_show (check);
+
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_ENABLE_REGEX,
+                                        check, "active");
 
   /* horizontal box for the replace all options */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+  gtk_widget_set_margin_start (GTK_WIDGET (hbox), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (hbox), 6);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
@@ -242,7 +315,8 @@ mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
   gtk_box_pack_start (GTK_BOX (hbox), check, FALSE, FALSE, 0);
   gtk_widget_show (check);
 
-  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_REPLACE_ALL, check, "active");
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_REPLACE_ALL,
+                                        check, "active");
 
   combo = dialog->search_location_combo = gtk_combo_box_text_new ();
   gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
@@ -252,7 +326,8 @@ mousepad_replace_dialog_init (MousepadReplaceDialog *dialog)
   gtk_widget_set_sensitive (combo, FALSE);
   gtk_widget_show (combo);
 
-  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_REPLACE_ALL_LOCATION, combo, "active");
+  mousepad_replace_dialog_bind_setting (dialog, MOUSEPAD_SETTING_SEARCH_REPLACE_ALL_LOCATION,
+                                        combo, "active");
 
   label = dialog->hits_label = gtk_label_new (NULL);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -304,21 +379,27 @@ mousepad_replace_dialog_response (GtkWidget *widget,
   const gchar           *search_str, *replace_str;
   gchar                 *message;
   gint                   search_direction, replace_all_location;
-  gboolean               match_case, match_whole_word, replace_all;
+  gboolean               wrap_around, match_case, enable_regex, match_whole_word, replace_all;
+
+  /* close dialog */
+  if (response_id == MOUSEPAD_RESPONSE_CLOSE || response_id < 0)
+    {
+      gtk_widget_destroy (widget);
+      return;
+    }
 
   /* read the search settings */
   search_direction = MOUSEPAD_SETTING_GET_INT (SEARCH_DIRECTION);
   replace_all_location = MOUSEPAD_SETTING_GET_INT (SEARCH_REPLACE_ALL_LOCATION);
+  wrap_around = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_WRAP_AROUND);
   match_case = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_MATCH_CASE);
+  enable_regex = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_ENABLE_REGEX);
   match_whole_word = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_MATCH_WHOLE_WORD);
   replace_all = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_REPLACE_ALL);
 
-  /* close dialog */
-  if (response_id == MOUSEPAD_RESPONSE_CLOSE)
-    goto destroy_dialog;
-
   /* search direction */
-  if (search_direction == DIRECTION_UP && ! replace_all)
+  if ((search_direction == DIRECTION_UP && response_id != MOUSEPAD_RESPONSE_REVERSE_FIND)
+      || (search_direction != DIRECTION_UP && response_id == MOUSEPAD_RESPONSE_REVERSE_FIND))
     flags = MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD;
   else
     flags = MOUSEPAD_SEARCH_FLAGS_DIR_FORWARD;
@@ -327,79 +408,61 @@ mousepad_replace_dialog_response (GtkWidget *widget,
   if (match_case)
     flags |= MOUSEPAD_SEARCH_FLAGS_MATCH_CASE;
 
+  /* enable regex search */
+  if (enable_regex)
+    flags |= MOUSEPAD_SEARCH_FLAGS_ENABLE_REGEX;
+
   /* only match whole words */
   if (match_whole_word)
     flags |= MOUSEPAD_SEARCH_FLAGS_WHOLE_WORD;
 
   /* wrap around */
-  if (search_direction == DIRECTION_BOTH && ! replace_all)
+  if (wrap_around)
     flags |= MOUSEPAD_SEARCH_FLAGS_WRAP_AROUND;
 
   /* search area */
-  if (replace_all && replace_all_location == IN_SELECTION)
-    flags |= MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION;
-  else
-    flags |= MOUSEPAD_SEARCH_FLAGS_AREA_DOCUMENT;
+  if (replace_all)
+    {
+      flags |= MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA;
+      if (replace_all_location == IN_ALL_DOCUMENTS)
+        flags |= MOUSEPAD_SEARCH_FLAGS_AREA_ALL_DOCUMENTS;
+      else if (replace_all_location == IN_SELECTION)
+        flags |= MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION;
+    }
 
   /* start position */
-  if (response_id == MOUSEPAD_RESPONSE_CHECK_ENTRY)
-    {
-      /* no visible actions */
-      flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_NONE;
-
-      if (replace_all)
-        goto replace_flags;
-      else
-        goto search_flags;
-    }
-  else if (response_id == MOUSEPAD_RESPONSE_FIND)
+  if (response_id == MOUSEPAD_RESPONSE_FIND || response_id == MOUSEPAD_RESPONSE_REVERSE_FIND)
     {
       /* select the first match */
       flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT;
 
-      search_flags:
-
       /* start at the 'end' of the selection */
       if (flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD)
         flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START;
       else
         flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_END;
     }
+  else if (response_id == MOUSEPAD_RESPONSE_ENTRY_CHANGED)
+    {
+      /* select the first match */
+      flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT;
+
+      /* start at the 'beginning' of the selection */
+      if (flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD)
+        flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_END;
+      else
+        flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START;
+    }
   else if (response_id == MOUSEPAD_RESPONSE_REPLACE)
     {
       /* replace matches */
       flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE;
 
-      if (replace_all)
-        {
-          replace_flags:
-
-          /* replace all from the beginning of the document */
-          flags |= MOUSEPAD_SEARCH_FLAGS_ITER_AREA_START
-                   | MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA;
-
-          /* search all opened documents (flag used in mousepad-window.c) */
-          if (replace_all_location == IN_ALL_DOCUMENTS)
-            flags |= MOUSEPAD_SEARCH_FLAGS_ALL_DOCUMENTS;
-        }
+      /* start at the 'beginning' of the selection */
+      if (flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD)
+        flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_END;
       else
-        {
-          /* start at the 'beginning' of the selection */
-          if (flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD)
-            flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_END;
-          else
-            flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START;
-        }
-    }
-  else
-    {
-      destroy_dialog:
-
-      /* destroy the window */
-      gtk_widget_destroy (widget);
-
-      /* leave */
-      return;
+        flags |= MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START;
     }
 
   /* get strings */
@@ -407,7 +470,8 @@ mousepad_replace_dialog_response (GtkWidget *widget,
   replace_str = gtk_entry_get_text (GTK_ENTRY (dialog->replace_entry));
 
   /* emit the signal */
-  g_signal_emit (G_OBJECT (dialog), dialog_signals[SEARCH], 0, flags, search_str, replace_str, &matches);
+  g_signal_emit (G_OBJECT (dialog), dialog_signals[SEARCH], 0,
+                 flags, search_str, replace_str, &matches);
 
   /* reset counter */
   if (response_id == MOUSEPAD_RESPONSE_REPLACE && replace_all)
@@ -419,7 +483,7 @@ mousepad_replace_dialog_response (GtkWidget *widget,
   /* update counter */
   if (replace_all)
     {
-      message = g_strdup_printf (ngettext ("%d occurence", "%d occurences", matches), matches);
+      message = g_strdup_printf (ngettext ("%d occurrence", "%d occurrences", matches), matches);
       gtk_label_set_markup (GTK_LABEL (dialog->hits_label), message);
       g_free (message);
     }
@@ -430,9 +494,9 @@ static void
 mousepad_replace_dialog_changed (MousepadReplaceDialog *dialog)
 {
   const gchar *text;
-  gboolean     sensitive;
   gboolean     replace_all;
 
+  dialog->sensitive = TRUE;
   replace_all = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_REPLACE_ALL);
 
   /* set the sensitivity of some dialog widgets */
@@ -445,29 +509,27 @@ mousepad_replace_dialog_changed (MousepadReplaceDialog *dialog)
   /* get the search entry text */
   text = gtk_entry_get_text (GTK_ENTRY (dialog->search_entry));
 
-  if (text != NULL && *text != '\0')
-    {
-      /* do an invisible search to give the user some visible feedback */
-      gtk_dialog_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_CHECK_ENTRY);
+  /* if the search entry text has changed, behaves like the search bar entry: find as you type
+   * else, it is likely to be an invisible search */
+  if (text != NULL)
+    gtk_dialog_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_ENTRY_CHANGED);
 
-      /* buttons are sensitive */
-      sensitive = TRUE;
-    }
-  else
+  /* update replace dialog */
+  if (text == NULL || *text == '\0')
     {
       /* not text, means no error */
       mousepad_util_entry_error (dialog->search_entry, FALSE);
 
-      /* reset occurences label */
+      /* reset occurrences label */
       gtk_label_set_text (GTK_LABEL (dialog->hits_label), NULL);
 
       /* buttons are not sensitive */
-      sensitive = FALSE;
+      dialog->sensitive = FALSE;
     }
 
   /* set the sensitivity */
-  gtk_widget_set_sensitive (dialog->find_button, sensitive);
-  gtk_widget_set_sensitive (dialog->replace_button, sensitive);
+  gtk_widget_set_sensitive (dialog->find_button, dialog->sensitive);
+  gtk_widget_set_sensitive (dialog->replace_button, dialog->sensitive);
 }
 
 
@@ -477,7 +539,7 @@ mousepad_replace_dialog_settings_changed (MousepadReplaceDialog *dialog,
                                           gchar                 *key,
                                           GSettings             *settings)
 {
-  /* reset occurences label */
+  /* reset occurrences label */
   gtk_label_set_text (GTK_LABEL (dialog->hits_label), NULL);
 
   mousepad_replace_dialog_changed (dialog);
@@ -485,6 +547,26 @@ mousepad_replace_dialog_settings_changed (MousepadReplaceDialog *dialog,
 
 
 
+static void
+mousepad_replace_dialog_entry_activate (GtkWidget             *entry,
+                                        MousepadReplaceDialog *dialog)
+{
+  if (dialog->sensitive)
+    gtk_dialog_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_FIND);
+}
+
+
+
+static void
+mousepad_replace_dialog_entry_reverse_activate (GtkWidget             *entry,
+                                                MousepadReplaceDialog *dialog)
+{
+  if (dialog->sensitive)
+    gtk_dialog_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_REVERSE_FIND);
+}
+
+
+
 /**
  * History functions
  **/
