@@ -15,7 +15,6 @@
  */
 
 #include <mousepad/mousepad-private.h>
-#include <mousepad/mousepad-gtkcompat.h>
 #include <mousepad/mousepad-settings.h>
 #include <mousepad/mousepad-marshal.h>
 #include <mousepad/mousepad-document.h>
@@ -28,7 +27,6 @@
 
 
 #define TOOL_BAR_ICON_SIZE  GTK_ICON_SIZE_MENU
-#define HIGHTLIGHT_TIMEOUT  225
 
 
 
@@ -46,9 +44,8 @@ static void      mousepad_search_bar_highlight_toggled          (GtkWidget
                                                                  MousepadSearchBar       *bar);
 static void      mousepad_search_bar_match_case_toggled         (GtkWidget               *button,
                                                                  MousepadSearchBar       *bar);
-static void      mousepad_search_bar_highlight_schedule         (MousepadSearchBar       *bar);
-static gboolean  mousepad_search_bar_highlight_timeout          (gpointer                 user_data);
-static void      mousepad_search_bar_highlight_timeout_destroy  (gpointer                 user_data);
+static void      mousepad_search_bar_enable_regex_toggled       (GtkWidget               *button,
+                                                                 MousepadSearchBar       *bar);
 
 
 
@@ -73,13 +70,12 @@ struct _MousepadSearchBar
 
   /* menu entries */
   GtkWidget           *match_case_entry;
+  GtkWidget           *enable_regex_entry;
 
   /* flags */
   guint                highlight_all : 1;
   guint                match_case : 1;
-
-  /* highlight id */
-  guint                highlight_id;
+  guint                enable_regex : 1;
 };
 
 
@@ -134,46 +130,24 @@ mousepad_search_bar_class_init (MousepadSearchBarClass *klass)
 
   /* setup key bindings for the search bar */
   binding_set = gtk_binding_set_by_class (klass);
-  gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0, "hide-bar", 0);
-
-  /* In GTK3, gtkrc is deprecated */
-#if GTK_CHECK_VERSION(3, 0, 0) && (__GNUC__ > 4 || __GNUC__ == 4 && __GNUC_MINOR__ > 2)
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
-
-  /* hide the shadow around the toolbar */
-  gtk_rc_parse_string ("style \"mousepad-search-bar-style\"\n"
-                         "{\n"
-                           "GtkToolbar::shadow-type = GTK_SHADOW_NONE\n"
-                         "}\n"
-                       "class \"MousepadSearchBar\" style \"mousepad-search-bar-style\"\n"
-
-                       /* add 2px space between the toolbar buttons */
-                       "style \"mousepad-button-style\"\n"
-                         "{\n"
-                           "GtkToolButton::icon-spacing = 2\n"
-                         "}\n"
-                       "widget \"MousepadWindow.*.Gtk*ToolButton\" style \"mousepad-button-style\"\n");
-
-#if GTK_CHECK_VERSION(3, 0, 0) && (__GNUC__ > 4 || __GNUC__ == 4 && __GNUC_MINOR__ > 2)
-# pragma GCC diagnostic pop
-#endif
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "hide-bar", 0);
 
   /* add an activate-backwards signal to GtkEntry */
   entry_class = g_type_class_ref (GTK_TYPE_ENTRY);
-  if (G_LIKELY (g_signal_lookup("activate-backward", GTK_TYPE_ENTRY) == 0))
+  if (G_LIKELY (g_signal_lookup ("activate-backward", GTK_TYPE_ENTRY) == 0))
     {
       /* install the signal */
-      g_signal_new("activate-backward",
+      g_signal_new ("activate-backward",
                    GTK_TYPE_ENTRY,
                    G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                    0, NULL, NULL,
                    g_cclosure_marshal_VOID__VOID,
                    G_TYPE_NONE, 0);
-      binding_set = gtk_binding_set_by_class(entry_class);
-      gtk_binding_entry_add_signal(binding_set, GDK_Return, GDK_SHIFT_MASK, "activate-backward", 0);
-      gtk_binding_entry_add_signal(binding_set, GDK_KP_Enter, GDK_SHIFT_MASK, "activate-backward", 0);
+      binding_set = gtk_binding_set_by_class (entry_class);
+      gtk_binding_entry_add_signal (binding_set, GDK_KEY_Return,
+                                    GDK_SHIFT_MASK, "activate-backward", 0);
+      gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Enter,
+                                    GDK_SHIFT_MASK, "activate-backward", 0);
     }
   g_type_class_unref (entry_class);
 }
@@ -183,33 +157,23 @@ mousepad_search_bar_class_init (MousepadSearchBarClass *klass)
 static void
 mousepad_search_bar_init (MousepadSearchBar *bar)
 {
-  GtkWidget   *label, *image, *check, *menuitem;
+  GtkWidget   *image, *check, *menuitem;
   GtkToolItem *item;
-  gboolean     match_case;
 
   /* load some saved state */
-  match_case = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_MATCH_CASE);
-
-  /* init variables */
-  bar->highlight_id = 0;
-  bar->match_case = match_case;
+  bar->match_case = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_MATCH_CASE);
+  bar->enable_regex = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_ENABLE_REGEX);
+  bar->highlight_all = MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_HIGHLIGHT_ALL);
 
   /* the close button */
-  item = gtk_tool_button_new_from_stock (GTK_STOCK_CLOSE);
+  image = gtk_image_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_BUTTON);
+  gtk_widget_show (image);
+  item = gtk_tool_button_new (image, NULL);
   gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
-  g_signal_connect_swapped (G_OBJECT (item), "clicked", G_CALLBACK (mousepad_search_bar_hide_clicked), bar);
+  g_signal_connect_swapped (G_OBJECT (item), "clicked",
+                            G_CALLBACK (mousepad_search_bar_hide_clicked), bar);
   gtk_widget_show (GTK_WIDGET (item));
 
-  /* the find label */
-  item = gtk_tool_item_new ();
-  gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
-  gtk_widget_show (GTK_WIDGET (item));
-
-  label = gtk_label_new_with_mnemonic (_("Fi_nd:"));
-  gtk_container_add (GTK_CONTAINER (item), label);
-  gtk_misc_set_padding (GTK_MISC (label), 2, 0);
-  gtk_widget_show (label);
-
   /* the entry field */
   item = gtk_tool_item_new ();
   gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
@@ -217,67 +181,106 @@ mousepad_search_bar_init (MousepadSearchBar *bar)
 
   bar->entry = gtk_entry_new ();
   gtk_container_add (GTK_CONTAINER (item), bar->entry);
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), bar->entry);
-  g_signal_connect (G_OBJECT (bar->entry), "changed", G_CALLBACK (mousepad_search_bar_entry_changed), bar);
-  g_signal_connect (G_OBJECT (bar->entry), "activate", G_CALLBACK (mousepad_search_bar_entry_activate), bar);
+  g_signal_connect (G_OBJECT (bar->entry), "changed",
+                    G_CALLBACK (mousepad_search_bar_entry_changed), bar);
+  g_signal_connect (G_OBJECT (bar->entry), "activate",
+                    G_CALLBACK (mousepad_search_bar_entry_activate), bar);
   g_signal_connect (G_OBJECT (bar->entry), "activate-backward", G_CALLBACK (mousepad_search_bar_entry_activate_backward), bar);
   gtk_widget_show (bar->entry);
 
-  /* next button */
-  image = gtk_image_new_from_stock (GTK_STOCK_GO_DOWN, TOOL_BAR_ICON_SIZE);
+  /* previous button */
+  image = gtk_image_new_from_icon_name ("go-up-symbolic", TOOL_BAR_ICON_SIZE);
   gtk_widget_show (image);
 
-  item = gtk_tool_button_new (image, _("_Next"));
-  gtk_tool_item_set_is_important (item, TRUE);
-  gtk_tool_button_set_use_underline (GTK_TOOL_BUTTON (item), TRUE);
+  item = gtk_tool_button_new (image, NULL);
   gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
-  g_signal_connect_swapped (G_OBJECT (item), "clicked", G_CALLBACK (mousepad_search_bar_find_next), bar);
+  g_signal_connect_swapped (G_OBJECT (item), "clicked",
+                            G_CALLBACK (mousepad_search_bar_find_previous), bar);
   gtk_widget_show (GTK_WIDGET (item));
 
-  /* previous button */
-  image = gtk_image_new_from_stock (GTK_STOCK_GO_UP, TOOL_BAR_ICON_SIZE);
+  /* next button */
+  image = gtk_image_new_from_icon_name ("go-down-symbolic", TOOL_BAR_ICON_SIZE);
   gtk_widget_show (image);
 
-  item = gtk_tool_button_new (image, _("_Previous"));
-  gtk_tool_item_set_is_important (item, TRUE);
-  gtk_tool_button_set_use_underline (GTK_TOOL_BUTTON (item), TRUE);
+  item = gtk_tool_button_new (image, NULL);
   gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
-  g_signal_connect_swapped (G_OBJECT (item), "clicked", G_CALLBACK (mousepad_search_bar_find_previous), bar);
+  g_signal_connect_swapped (G_OBJECT (item), "clicked",
+                            G_CALLBACK (mousepad_search_bar_find_next), bar);
   gtk_widget_show (GTK_WIDGET (item));
 
   /* highlight all */
   item = (GtkToolItem *) gtk_toggle_tool_button_new ();
-  g_signal_connect_object (G_OBJECT (bar), "destroy", G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
-  gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), GTK_STOCK_SELECT_ALL);
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), "edit-select-all-symbolic");
   gtk_tool_button_set_label (GTK_TOOL_BUTTON (item), _("Highlight _All"));
   gtk_tool_item_set_is_important (item, TRUE);
   gtk_tool_button_set_use_underline (GTK_TOOL_BUTTON (item), TRUE);
   gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
-  g_signal_connect (G_OBJECT (item), "clicked", G_CALLBACK (mousepad_search_bar_highlight_toggled), bar);
+  g_signal_connect (G_OBJECT (item), "clicked",
+                    G_CALLBACK (mousepad_search_bar_highlight_toggled), bar);
   gtk_widget_show (GTK_WIDGET (item));
 
+  MOUSEPAD_SETTING_BIND (SEARCH_HIGHLIGHT_ALL, item, "active", G_SETTINGS_BIND_DEFAULT);
+
   /* check button for case sensitive, including the proxy menu item */
   item = gtk_tool_item_new ();
-  g_signal_connect_object (G_OBJECT (bar), "destroy", G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
   gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
   gtk_widget_show (GTK_WIDGET (item));
 
-  check = gtk_check_button_new_with_mnemonic (_("Mat_ch Case"));
-  g_signal_connect_object (G_OBJECT (bar), "destroy", G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  check = gtk_check_button_new_with_mnemonic (_("Match _case"));
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
   gtk_container_add (GTK_CONTAINER (item), check);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), match_case);
-  g_signal_connect (G_OBJECT (check), "toggled", G_CALLBACK (mousepad_search_bar_match_case_toggled), bar);
+  g_signal_connect (G_OBJECT (check), "toggled",
+                    G_CALLBACK (mousepad_search_bar_match_case_toggled), bar);
   gtk_widget_show (check);
 
   /* keep the widgets in sync with the GSettings */
   MOUSEPAD_SETTING_BIND (SEARCH_MATCH_CASE, check, "active", G_SETTINGS_BIND_DEFAULT);
 
   /* overflow menu item for when window is too narrow to show the tool bar item */
-  bar->match_case_entry = menuitem = gtk_check_menu_item_new_with_mnemonic (_("Mat_ch Case"));
-  g_signal_connect_object (G_OBJECT (bar), "destroy", G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  bar->match_case_entry = menuitem
+                        = gtk_check_menu_item_new_with_mnemonic (_("Match _case"));
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
   gtk_tool_item_set_proxy_menu_item (item, "case-sensitive", menuitem);
+
   /* Keep toolbar check button and overflow proxy menu item in sync */
-  g_object_bind_property (check, "active", menuitem, "active", G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+  g_object_bind_property (check, "active", menuitem, "active",
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+  gtk_widget_show (menuitem);
+
+  /* check button for enabling regex, including the proxy menu item */
+  item = gtk_tool_item_new ();
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  gtk_toolbar_insert (GTK_TOOLBAR (bar), item, -1);
+  gtk_widget_show (GTK_WIDGET (item));
+
+  check = gtk_check_button_new_with_mnemonic (_("Regular e_xpression"));
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  gtk_container_add (GTK_CONTAINER (item), check);
+  g_signal_connect (G_OBJECT (check), "toggled",
+                    G_CALLBACK (mousepad_search_bar_enable_regex_toggled), bar);
+  gtk_widget_show (check);
+
+  /* keep the widgets in sync with the GSettings */
+  MOUSEPAD_SETTING_BIND (SEARCH_ENABLE_REGEX, check, "active", G_SETTINGS_BIND_DEFAULT);
+
+  /* overflow menu item for when window is too narrow to show the tool bar item */
+  bar->enable_regex_entry = menuitem
+                          = gtk_check_menu_item_new_with_mnemonic (_("Regular e_xpression"));
+  g_signal_connect_object (G_OBJECT (bar), "destroy",
+                           G_CALLBACK (gtk_widget_destroy), item, G_CONNECT_SWAPPED);
+  gtk_tool_item_set_proxy_menu_item (item, "enable-regex", menuitem);
+
+  /* Keep toolbar check button and overflow proxy menu item in sync */
+  g_object_bind_property (check, "active", menuitem, "active",
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
   gtk_widget_show (menuitem);
 }
 
@@ -286,12 +289,6 @@ mousepad_search_bar_init (MousepadSearchBar *bar)
 static void
 mousepad_search_bar_finalize (GObject *object)
 {
-  MousepadSearchBar *bar = MOUSEPAD_SEARCH_BAR (object);
-
-  /* stop a running highlight timeout */
-  if (bar->highlight_id != 0)
-    g_source_remove (bar->highlight_id);
-
   (*G_OBJECT_CLASS (mousepad_search_bar_parent_class)->finalize) (object);
 }
 
@@ -304,26 +301,27 @@ mousepad_search_bar_find_string (MousepadSearchBar   *bar,
   const gchar *string;
   gint         nmatches;
 
-  /* search the entire document */
-  flags |= MOUSEPAD_SEARCH_FLAGS_AREA_DOCUMENT;
-
-  /* if we don't hightlight, we select with wrapping */
-  if ((flags & MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHTLIGHT) == 0)
-    flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT
-             | MOUSEPAD_SEARCH_FLAGS_WRAP_AROUND;
+  /* always true when using the search bar */
+  flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT
+           | MOUSEPAD_SEARCH_FLAGS_WRAP_AROUND;
 
   /* append the insensitive flags when needed */
   if (bar->match_case)
     flags |= MOUSEPAD_SEARCH_FLAGS_MATCH_CASE;
 
+  /* append the regex flags when needed */
+  if (bar->enable_regex)
+    flags |= MOUSEPAD_SEARCH_FLAGS_ENABLE_REGEX;
+
   /* get the entry string */
   string = gtk_entry_get_text (GTK_ENTRY (bar->entry));
 
   /* emit signal */
   g_signal_emit (G_OBJECT (bar), search_bar_signals[SEARCH], 0, flags, string, NULL, &nmatches);
 
-  /* do nothing with the error entry when highlight when trigged with highlight */
-  if ((flags & MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHTLIGHT) == 0)
+  /* do nothing with the entry error when triggered by highlight */
+  if (! (flags & (MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHLIGHT_ON
+                  | MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHLIGHT_OFF)))
     {
       /* make sure the search entry is not red when no text was typed */
       if (string == NULL || *string == '\0')
@@ -377,9 +375,6 @@ mousepad_search_bar_entry_changed (GtkWidget         *entry,
 
   /* find */
   mousepad_search_bar_find_string (bar, flags);
-
-  /* schedule a new highlight */
-  mousepad_search_bar_highlight_schedule (bar);
 }
 
 
@@ -392,27 +387,15 @@ mousepad_search_bar_highlight_toggled (GtkWidget         *button,
 
   g_return_if_fail (MOUSEPAD_IS_SEARCH_BAR (bar));
 
-  /* set the new state */
+  /* set the new state and search flags */
   bar->highlight_all = gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (button));
-
   if (bar->highlight_all)
-    {
-      /* reschedule the highlight */
-      mousepad_search_bar_highlight_schedule (bar);
-    }
+    flags = MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHLIGHT_ON;
   else
-    {
-      /* stop timeout */
-      if (bar->highlight_id != 0)
-        g_source_remove (bar->highlight_id);
+    flags = MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHLIGHT_OFF;
 
-      /* set search flags */
-      flags = MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHTLIGHT
-              | MOUSEPAD_SEARCH_FLAGS_ACTION_CLEANUP;
-
-      /* emit signal to cleanup the highlight */
-      mousepad_search_bar_find_string (bar, flags);
-    }
+  /* emit signal to set the highlight */
+  mousepad_search_bar_find_string (bar, flags);
 }
 
 
@@ -433,56 +416,26 @@ mousepad_search_bar_match_case_toggled (GtkWidget         *button,
 
   /* search ahead with this new flags */
   mousepad_search_bar_entry_changed (NULL, bar);
-
-  /* schedule a new hightlight */
-  mousepad_search_bar_highlight_schedule (bar);
 }
 
 
 
 static void
-mousepad_search_bar_highlight_schedule (MousepadSearchBar *bar)
+mousepad_search_bar_enable_regex_toggled (GtkWidget         *button,
+                                          MousepadSearchBar *bar)
 {
+  gboolean active;
+
   g_return_if_fail (MOUSEPAD_IS_SEARCH_BAR (bar));
 
-  /* stop a pending timeout */
-  if (bar->highlight_id != 0)
-    g_source_remove (bar->highlight_id);
+  /* get the state of the toggle button */
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 
-  /* schedule a new timeout */
-  if (bar->highlight_all)
-    {
-      bar->highlight_id = g_timeout_add_full (G_PRIORITY_LOW, HIGHTLIGHT_TIMEOUT, mousepad_search_bar_highlight_timeout,
-                                              bar, mousepad_search_bar_highlight_timeout_destroy);
-    }
-}
+  /* save the state */
+  bar->enable_regex = active;
 
-
-
-static void
-mousepad_search_bar_highlight_timeout_destroy (gpointer user_data)
-{
-  MOUSEPAD_SEARCH_BAR (user_data)->highlight_id = 0;
-}
-
-
-
-static gboolean
-mousepad_search_bar_highlight_timeout (gpointer user_data)
-{
-  MousepadSearchBar   *bar = MOUSEPAD_SEARCH_BAR (user_data);
-  MousepadSearchFlags  flags;
-
-  /* set search flags */
-  flags = MOUSEPAD_SEARCH_FLAGS_DIR_FORWARD
-          | MOUSEPAD_SEARCH_FLAGS_ITER_AREA_START
-          | MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHTLIGHT;
-
-  /* emit signal */
-  mousepad_search_bar_find_string (bar, flags);
-
-  /* stop the timeout */
-  return FALSE;
+  /* search ahead with this new flags */
+  mousepad_search_bar_entry_changed (NULL, bar);
 }
 
 
@@ -501,19 +454,22 @@ mousepad_search_bar_entry (MousepadSearchBar *bar)
 void
 mousepad_search_bar_focus (MousepadSearchBar *bar)
 {
+  MousepadSearchFlags flags;
+
   g_return_if_fail (MOUSEPAD_IS_SEARCH_BAR (bar));
 
   /* focus the entry field */
   gtk_widget_grab_focus (bar->entry);
 
-  /* trigger search function */
-  mousepad_search_bar_entry_changed (NULL, bar);
-
-  /* update the highlight */
-  mousepad_search_bar_highlight_schedule (bar);
-
   /* select the entire entry */
   gtk_editable_select_region (GTK_EDITABLE (bar->entry), 0, -1);
+
+  /* highlighting has been disabled by hiding the search bar */
+  if (bar->highlight_all)
+    {
+      flags = MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHLIGHT_ON;
+      mousepad_search_bar_find_string (bar, flags);
+    }
 }
 
 
