@@ -18,6 +18,7 @@
 #define __MOUSEPAD_WINDOW_H__
 
 #include <mousepad/mousepad-document.h>
+#include <mousepad/mousepad-application.h>
 
 G_BEGIN_DECLS
 
@@ -43,18 +44,21 @@ static const GtkTargetEntry drop_targets[] =
 typedef struct _MousepadWindowClass MousepadWindowClass;
 typedef struct _MousepadWindow      MousepadWindow;
 
-GType           mousepad_window_get_type         (void) G_GNUC_CONST;
+GType           mousepad_window_get_type                   (void) G_GNUC_CONST;
 
-GtkWidget      *mousepad_window_new              (void);
+GtkWidget      *mousepad_window_new                        (MousepadApplication  *application);
 
-void            mousepad_window_add              (MousepadWindow   *window,
-                                                  MousepadDocument *document);
+void            mousepad_window_add                        (MousepadWindow       *window,
+                                                            MousepadDocument     *document);
 
-gboolean        mousepad_window_open_files       (MousepadWindow  *window,
-                                                  const gchar     *working_directory,
-                                                  gchar          **filenames);
+gint            mousepad_window_open_files                 (MousepadWindow       *window,
+                                                            gchar               **uris);
 
-void            mousepad_window_show_preferences (MousepadWindow  *window);
+void            mousepad_window_show_preferences           (MousepadWindow       *window);
+
+GtkWidget      *mousepad_window_get_languages_menu         (MousepadWindow       *window);
+
+void            mousepad_window_update_document_menu_items (MousepadWindow       *window);
 
 G_END_DECLS
 
